﻿using System;
using Monocle;
using On.Celeste;

namespace Celeste.Mod.MegaHotMod
{
	public class MegaHotModule : EverestModule
	{
		public static MegaHotModule Instance;

		public MegaHotModule()
        {
            Instance = this;
        }

		public override Type SettingsType => typeof(MegaHotSettings);
		MegaHotSettings Settings => (MegaHotSettings)Instance._Settings;
		public override Type SaveDataType => null;      

		private float averageTimeRate = 1f;

		public override void Load()
		{
			On.Celeste.Player.Update += PlayerUpdate;
			On.Celeste.Level.LoadLevel += Level_LoadLevel;
		}

		private void Level_LoadLevel(On.Celeste.Level.orig_LoadLevel orig, Level self, Player.IntroTypes playerIntro, bool isFromLoader)
		{
			orig(self, playerIntro, isFromLoader);
			this.averageTimeRate = 1f;
		}

		private void PlayerUpdate(On.Celeste.Player.orig_Update orig, Player self)
		{
			orig(self);
			if (self.InControl && !self.Dead && !self.IsIntroState)
			{
				this.averageTimeRate = Math.Min(
					Math.Max(
						(this.averageTimeRate * (float)Settings.LerpSmoothness
						 + 0.2f 
						 + (
							(float)Math.Abs(Input.MoveY.Value) 
						 	+ (float)Math.Abs(Input.MoveX.Value) 
						 	+ self.Speed.Length()/310f 
						 	+ (Input.Dash.Pressed ? 0.2f : 0f) 
						 	+ (Input.Jump.Pressed ? 0.2f : 0f)
							)*1.2f
						) / (1f+Settings.LerpSmoothness),
						Settings.LerpLowPoint/10
					),
				Settings.LerpHighPoint/10);
			}
			else
			{
				this.averageTimeRate = (this.averageTimeRate * 2f + 1f) / 3f;
			}

			if (Settings.Enabled)
			{
				Engine.TimeRateB = this.averageTimeRate;
			}
		}

		public override void Unload()
		{
			On.Celeste.Player.Update -= PlayerUpdate;
			On.Celeste.Level.LoadLevel -= Level_LoadLevel;
		}
	}
}
